/* c-basic-offset: 2; tab-width: 2; indent-tabs-mode: nil
 * vi: set shiftwidth=2 tabstop=2 expandtab:
 * :indentSize=2:tabSize=2:noTabs=true:
 */

/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "DynamicArray.h"

struct DynamicArray {
  size_t max_size;
  size_t len;
  FreeFunc free_func;
  spointer * array;
};

void _private_for_each_item_free (DynamicArray * self, spointer item,
                                    spointer data);

DynamicArray * dynamic_array_new (size_t len, FreeFunc free_func) {
  DynamicArray * self = malloc (sizeof(DynamicArray));
  
  self->max_size = len;
  self->len = 0;
  
  self->free_func = free_func;
  
  self->array = malloc (len * sizeof (* self->array));
  
  return self;
}

void dynamic_array_free (DynamicArray * self) {
  if (self->free_func != NULL) {
    dynamic_array_for_each (self, (ForEachFunc) _private_for_each_item_free,
                            NULL);
  } else {
    for (int i = 0; i < self->len; i++) {
      free (self->array[i]);
    }
  }
  free (self->array);
  free (self);
}

spointer dynamic_array_get (DynamicArray * self, size_t index) {
  return self->array[index];
}

size_t dynamic_array_len (DynamicArray * self) {
  return self->len;
}

size_t dynamic_array_size (DynamicArray * self) {
  return self->max_size;
}

void dynamic_array_add (DynamicArray * self, spointer data) {
  if (self->len >= self->max_size - 1) {
    self->array = realloc (self->array,
                          (sizeof (* self->array) *
                          (self->max_size + ARRAY_PADDING)));
    self->max_size = self->max_size + ARRAY_PADDING;
  }
  self->array[self->len] = data;
  self->len++;
}

spointer * dynamic_array_dump_array (DynamicArray * self) {
  spointer * ret_val = malloc (sizeof (* self->array) * (self->len + 1));
  for (int i = 0; i < self->len; i++) {
    ret_val[i] = self->array[i];
  }
  ret_val[self->len] = NULL;
  return ret_val;
}

void dynamic_array_for_each (DynamicArray * self, ForEachFunc func,
                             spointer data) {
  for (int i = 0; i < self->len; i++) {
    func (self, self->array[i], data);
  }
}


void _private_for_each_item_free (DynamicArray * self, spointer item,
                                    spointer data) {
  FreeFunc func = self->free_func;
  func (item);
}

