/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

/** @file */

#ifndef __H_ERROR__
#define __H_ERROR__

#include "baseobject.h"
#include "defs.h"
#include "utils.h"
#include "Func.h"
#include <limits.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

BEGIN_DECLS

/**
 * An opaque datastructure that holds the SError's private data.
 */
typedef struct SErrorPrivate SErrorPrivate;


/**
 * An SError is a data structure that inherits from SBaseObjectInstance.
 * 
 * An SError represents an error that can occur.
 */
typedef struct SError {
  SObject parent;
  SErrorPrivate * priv; /** Pimple pointer to the private data. */
  
} SError;

/**
 * The error class is not changed, no extra methods are needed.
 */
typedef struct SErrorClass {
  SObjectClass parent_class;
  
} SErrorClass;

/** @brief
 * The different error types.
 */
typedef enum {
  S_ERROR_NONE = 0, /**< Not on error */
  S_ERROR_NULL = 1, /**< An NULL error */
  S_ERROR_OTHER = 2, /**< Some unknowned error */
  S_ERROR_INPUT_OUTPUT = 3, /**< An I/O error */
  S_ERROR_OVERFLOW = 4, /**< An Overflow error */
} SErrorType;

/**
 * Holds the names of the string version of SErrorType.
 */
static char * SErrorTypeName[] = {
  "NONE",
  "NULL",
  "OTHER",
  "INPUT/OUTPUT",
  "OVERFLOW",
  NULL,
  NULL
};


/**
 * the constructor for the an SError
 */
SError * s_error_new (SErrorType error, char * message);

/**
 * This function calles the deinitize method of the object.
 */
void s_error_free (SError * self);

/**
 * This function returns the ErrorType of on object.
 */
SErrorType s_error_get_error_type (SError * self);

/**
 * This function prints the current error to stdout.
 */
void s_error_print_error (SError * self);

END_DECLS

#endif
