/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "LinkedList.h"
#include "stdio.h"


struct LinkedListNode {
  LinkedListNode * prev;
  LinkedListNode * next;
  void * data;
};

struct LinkedList {
  size_t len; /**< The length of the list, may be wrong.*/
  void (* free_func)(void *); /**< The function that frees the data. */
  LinkedListNode * head; /**< head->prev must be NULL */
  LinkedListNode * tail; /**< tail->next must be NULL */
  LinkedListNode * current; /**< Represents the curret object*/
};


LinkedList * linked_list_new (void (* free_func)(void *)) {
  LinkedList * self = malloc (sizeof (LinkedList));
  self->len = 0;
  self->free_func = free_func;
  self->head = self->tail = self->current = NULL;
  return self;
}


void linked_list_free (LinkedList * self, bool free_data) {
  
}


void linked_list_add (LinkedList * self, void * data) {
  if (self->len  == 0) { // special case when the list is empty.
    self->head = self->tail = self->current = malloc (sizeof (LinkedListNode));
    self->tail->prev = NULL;
    self->head->next = NULL;
    self->current->data = data;
  } else { // general case.
    self->tail->next = malloc (sizeof (LinkedListNode));
    self->tail->next->next = NULL;
  }
}

void linked_list_next (LinkedList * self) {
  if (self->current->next) {
    self->current = self->current->next;
  } else {
    fprintf (stderr, "Reached end of list %lld\n", (long long int) self);
    assert (self->current == self->tail);
  }
}


void linked_list_pev (LinkedList * self) {
  if (self->current->prev) {
    self->current = self->current->prev;
  } else {
    fprintf (stderr, "Reached begining of list %lld\n", (long long int) self);
    assert (self->current == self->head);
  }
}


void * linked_list_get_current (LinkedList * self) {
  return self->current->data;
}


void * linked_list_get_next (LinkedList * self) {
  if (self->current->next) {
    return self->current->next;
  } else {
    fprintf (stderr, "No \"next\" item exists in list %lld\n", (long long int)
                                                                self);
    assert (self->current == self->tail);
    return NULL;
  }
}


void * linked_list_get_prev (LinkedList * self) {
    if (self->current->prev) {
    return self->current->next;
  } else {
    fprintf (stderr, "No \"previous\" item exists in list %lld\n",
            (long long int) self);
    assert (self->current == self->head);
    return NULL;
  }
}


void linked_list_head (LinkedList * self) {
  self->current = self->head;
}


void linked_list_tail (LinkedList * self) {
  self->current = self->tail;
}


size_t linked_list_len (LinkedList * self)  {
  return self->len;
}

void linked_list_remove_current (LinkedList * self) {
  // TODO
}


