/*
Copyright (c) 2013-2014 Gustav Hartvigsson

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef __H_UTILS__
#define __H_UTILS__

#include "defs.h"
#include <stddef.h>
#include "config.h"
#include <stdio.h>

BEGIN_DECLS

/**
 * Creats a new C string with the correct length from a long string.
 * This may be a costly operation.
 *
 * The restulting string must be freed by caller.
 */
char * s_string_new (const char * s);

/**
 * Creats a new C string with the correct length using standard fprint style
 * format.
 *
 * The restulting string must be freed by caller.
 */
char * s_string_new_fmt (const char * format, ...);

/**
 * Same as s_string_new, but with a set length.
 *
 * The restulting string must be freed by caller
 */
char * s_string_new_with_len (const char * s, size_t len);

/**
 * Returns a C string with the current time (as run).
 *
 * The restulting string must be freed by caller.
 */
char * s_current_time (void);

#ifndef strdup
/* strdup is not ISO C, so we have to declare it somewhare, this should work
 * even if we do not impliment the function ourselfs.
 *
 * This just to supress a compiler warning.
 */
char * strdup(const char *str);
#endif

/** Just a standard print function. */
#define s_print(p, ...)\
  fprintf (stdout, p, ##__VA_ARGS__)

/**
 * Print a yellow warning to stderr.
 */
#define s_warn_print(p, ...)\
  fprintf (stderr, BOLDYELLOW "[WARN] " p "\n" RESET, ##__VA_ARGS__)

/**
 * prints a red message with the prefix [ERR]
 */
#define s_err_print(p, ...)\
  fprintf (stderr, RED "[ERR] " p RESET "\n", ##__VA_ARGS__)


#if DEBUG
/**
 * debug_print is a function that only prints if compiled with the debug
 * flag not unset.
 */
#  define s_dbg_print(M, ...)\
    fprintf (stderr, YELLOW "[DEBUG] " M RESET "\n", ##__VA_ARGS__)
#else
#  define s_dbg_print(M, ...) 
#endif

BEGIN_DECLS

#endif
