
#ifndef __H_VEC__
#define __H_VEC__

#include <stdlib.h>
#include <stdint.h>
#include "defs.h"

/** @file
 * Contains the definitions of Vectors.
 */

BEGIN_DECLS

/** @brief
 * A Vector of three standard int values.
 *
 * A Vec3 is considerer a primitive type and as such has no ref/unref memory
 * management and no vec_free function. You will have to do that yourself.
 *
 * @sa vec3_new
 * @sa Vec3_16
 */
typedef struct Vec3 {
  int a;
  int b;
  int c;
} Vec3;

/** @brief
 * allocates and initialises a Vec3 pointer.
 *
 * @sa Vec3
 */
Vec3 * vec3_new (int a, int b, int c);

/** @brief
 * A Vector of three 16 bit int values.
 *
 * @sa vec3_16_new
 * @sa Vec3
 */
typedef struct Vec3_16 {
  int16_t a;
  int16_t b;
  int16_t c;
} Vec3_16;

/** @brief
 * allocates and initialises a Vec3_16 pointer.
 * 
 * @sa Vec3_16
 */
Vec3_16 * vec3_16_new (int16_t a, int16_t b, int16_t c);

END_DECLS

#endif /* __H_VEC__ */
